/*
 * Copyright (C) 2001 Karsten Reincke <karsten.reincke@gnukose.org>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 * file <muo_convert_table.cc> version <#1.3.1#> of project <MrProjext>
 */   

/** \file muo_convert_table.cc
 *  \brief all definitions of and for the conversion of strings
 *
 *  \author karsten reincke
 */

#include <iostream>
#include <string>
#include <assert.h>
#include "muo_convert_table.h"

/* &&& (1) local preprocessor-defines &&&&&&&&&&&&&&&&&&&&&&&& */

/* &&& (2) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* :-: (2.A) public constructors, inits & operators -::--:-:-: */
/** 
 *  \brief constructor for class CodeTokenMap
 *
 *  normally it hasn't to do anything
 */
krmuo::CodeTokenMap::CodeTokenMap()
: mClearToken(NULL),
  mDecoderMap(NULL),
  mEncoderMap(NULL)
{
}
           
/** 
 *  \brief destructor for class CodeTokenMap
 *
 *  normally it hasn't to do anything
 */
       
krmuo::CodeTokenMap::~CodeTokenMap()
{
}
/* :-: (2.B) public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/**
 *  \brief is the given sign encodable?
 *
 *  \param pSign = sign, which shall be encoded
 *  \return true if it is an encodable sign otherwise false
 */
bool krmuo::CodeTokenMap::isEncodableSign(unsigned char pSign) const
{
  
  if (mEncoderMap->find(pSign) == mEncoderMap->end())
    return false;
  else
    return true;
}

/**
 *  \brief encode the given sign
 *
 *  \param pSign = sign, which shall be encoded
 *  \return result of the encodation (pointer to the HtmlCodeTokenMap
 */
const string& krmuo::CodeTokenMap::encodeSign(unsigned char pSign) const
{
  map<unsigned char,string>::const_iterator 
    encodationTupel=mEncoderMap->find(pSign);
  
  if (encodationTupel == mEncoderMap->end())
    return *mClearToken;
  else
    return (*encodationTupel).second;
}
    
/**
 *  \brief is the given token decodable ?
 *
 *  \param pToken = string, which shall be decoded
 *  \return true if it is an decodable string otherwise false
 */
bool krmuo::CodeTokenMap::isDecodableToken(const string& pToken) const
{  
  if (mDecoderMap->find(pToken) == mDecoderMap->end())
    return false;
  else
    return true;
}

/**
 *  \brief returns sign for which the given entity stands
 *
 *  \param pToken = string, which shall be decoded
 *  \return result of the decodation
 */
unsigned char krmuo::CodeTokenMap::decodeToken(const string& pToken) const
{
  map<string,unsigned char>::const_iterator 
    decodationTupel=mDecoderMap->find(pToken);
  
  if (decodationTupel == mDecoderMap->end())
    return ' ';
  else
    return (*decodationTupel).second;
}


/* &&& (3) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&& (3) definitions of static global variables &&&&&&&&&&& */
const string krmuo::HtmlCodeTokenMap::mHtmlClearToken="";
map<string,unsigned char> krmuo::HtmlCodeTokenMap::mHtmlDecoderMap;
map<unsigned char,string> krmuo::HtmlCodeTokenMap::mHtmlEncoderMap;
bool krmuo::HtmlCodeTokenMap::mHtmlMapsMustStillBeFilled=true;

const unsigned char krmuo::HtmlCodeTokenMap::signValueAmp=(unsigned char)'&';
       const string krmuo::HtmlCodeTokenMap::signEncodationAmp="&amp;";
  
const unsigned char krmuo::HtmlCodeTokenMap::signValueLt=(unsigned char)'<';
       const string krmuo::HtmlCodeTokenMap::signEncodationLt="&lt;";

const unsigned char krmuo::HtmlCodeTokenMap::signValueGt=(unsigned char)'>';
       const string krmuo::HtmlCodeTokenMap::signEncodationGt="&gt;";

const unsigned char krmuo::HtmlCodeTokenMap::signValueApos=(unsigned char)'\'';
       const string krmuo::HtmlCodeTokenMap::signEncodationApos="&apos;";

const unsigned char krmuo::HtmlCodeTokenMap::signValueQuot=(unsigned char)'\"';
       const string krmuo::HtmlCodeTokenMap::signEncodationQuot="&quot;";

const unsigned char krmuo::HtmlCodeTokenMap::signValue160=(unsigned char)160;
       const string krmuo::HtmlCodeTokenMap::signEncodation160="&nbsp;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue161=(unsigned char)161;
       const string krmuo::HtmlCodeTokenMap::signEncodation161="&iexcl;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue162=(unsigned char)162;
       const string krmuo::HtmlCodeTokenMap::signEncodation162="&cent;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue163=(unsigned char)163;
       const string krmuo::HtmlCodeTokenMap::signEncodation163="&pound;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue164=(unsigned char)164;
       const string krmuo::HtmlCodeTokenMap::signEncodation164="&curren;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue165=(unsigned char)165;
       const string krmuo::HtmlCodeTokenMap::signEncodation165="&yen;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue166=(unsigned char)166;
       const string krmuo::HtmlCodeTokenMap::signEncodation166="&brvbar;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue167=(unsigned char)167;
       const string krmuo::HtmlCodeTokenMap::signEncodation167="&sect;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue168=(unsigned char)168;
       const string krmuo::HtmlCodeTokenMap::signEncodation168="&uml;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue169=(unsigned char)169;
       const string krmuo::HtmlCodeTokenMap::signEncodation169="&copy;";

const unsigned char krmuo::HtmlCodeTokenMap::signValue170=(unsigned char)170;
       const string krmuo::HtmlCodeTokenMap::signEncodation170="&ordf;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue171=(unsigned char)171;
       const string krmuo::HtmlCodeTokenMap::signEncodation171="&laquo;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue172=(unsigned char)172;
       const string krmuo::HtmlCodeTokenMap::signEncodation172="&not;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue173=(unsigned char)173;
       const string krmuo::HtmlCodeTokenMap::signEncodation173="&shy;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue174=(unsigned char)174;
       const string krmuo::HtmlCodeTokenMap::signEncodation174="&reg;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue175=(unsigned char)175;
       const string krmuo::HtmlCodeTokenMap::signEncodation175="&macr;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue176=(unsigned char)176;
       const string krmuo::HtmlCodeTokenMap::signEncodation176="&deg;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue177=(unsigned char)177;
       const string krmuo::HtmlCodeTokenMap::signEncodation177="&plusmn;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue178=(unsigned char)178;
       const string krmuo::HtmlCodeTokenMap::signEncodation178="&sup2;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue179=(unsigned char)179;
       const string krmuo::HtmlCodeTokenMap::signEncodation179="&sup3;";

const unsigned char krmuo::HtmlCodeTokenMap::signValue180=(unsigned char)180;
       const string krmuo::HtmlCodeTokenMap::signEncodation180="&acute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue181=(unsigned char)181;
       const string krmuo::HtmlCodeTokenMap::signEncodation181="&micro;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue182=(unsigned char)182;
       const string krmuo::HtmlCodeTokenMap::signEncodation182="&para;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue183=(unsigned char)183;
       const string krmuo::HtmlCodeTokenMap::signEncodation183="&middot;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue184=(unsigned char)184;
       const string krmuo::HtmlCodeTokenMap::signEncodation184="&cedil;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue185=(unsigned char)185;
       const string krmuo::HtmlCodeTokenMap::signEncodation185="&sup1;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue186=(unsigned char)186;
       const string krmuo::HtmlCodeTokenMap::signEncodation186="&ordm;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue187=(unsigned char)187;
       const string krmuo::HtmlCodeTokenMap::signEncodation187="&raquo;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue188=(unsigned char)188;
       const string krmuo::HtmlCodeTokenMap::signEncodation188="&frac14;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue189=(unsigned char)189;
       const string krmuo::HtmlCodeTokenMap::signEncodation189="&frac12;";

const unsigned char krmuo::HtmlCodeTokenMap::signValue190=(unsigned char)190;
       const string krmuo::HtmlCodeTokenMap::signEncodation190="&frac34;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue191=(unsigned char)191;
       const string krmuo::HtmlCodeTokenMap::signEncodation191="&iquest;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue192=(unsigned char)192;
       const string krmuo::HtmlCodeTokenMap::signEncodation192="&Agrave;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue193=(unsigned char)193;
       const string krmuo::HtmlCodeTokenMap::signEncodation193="&Aacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue194=(unsigned char)194;
       const string krmuo::HtmlCodeTokenMap::signEncodation194="&Acirc;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue195=(unsigned char)195;
       const string krmuo::HtmlCodeTokenMap::signEncodation195="&Atilde;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue196=(unsigned char)196;
       const string krmuo::HtmlCodeTokenMap::signEncodation196="&Auml;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue197=(unsigned char)197;
       const string krmuo::HtmlCodeTokenMap::signEncodation197="&Aring;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue198=(unsigned char)198;
       const string krmuo::HtmlCodeTokenMap::signEncodation198="&AElig;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue199=(unsigned char)199;
       const string krmuo::HtmlCodeTokenMap::signEncodation199="&Ccedil;";

const unsigned char krmuo::HtmlCodeTokenMap::signValue200=(unsigned char)200;
       const string krmuo::HtmlCodeTokenMap::signEncodation200="&Egrave;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue201=(unsigned char)201;
       const string krmuo::HtmlCodeTokenMap::signEncodation201="&Eacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue202=(unsigned char)202;
       const string krmuo::HtmlCodeTokenMap::signEncodation202="&Ecirc;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue203=(unsigned char)203;
       const string krmuo::HtmlCodeTokenMap::signEncodation203="&Euml;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue204=(unsigned char)204;
       const string krmuo::HtmlCodeTokenMap::signEncodation204="&Igrave;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue205=(unsigned char)205;
       const string krmuo::HtmlCodeTokenMap::signEncodation205="&Iacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue206=(unsigned char)206;
       const string krmuo::HtmlCodeTokenMap::signEncodation206="&Icirc;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue207=(unsigned char)207;
       const string krmuo::HtmlCodeTokenMap::signEncodation207="&Iuml;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue208=(unsigned char)208;
       const string krmuo::HtmlCodeTokenMap::signEncodation208="&ETH;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue209=(unsigned char)209;
       const string krmuo::HtmlCodeTokenMap::signEncodation209="&Ntilde;";

const unsigned char krmuo::HtmlCodeTokenMap::signValue210=(unsigned char)210;
       const string krmuo::HtmlCodeTokenMap::signEncodation210="&Ograve;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue211=(unsigned char)211;
       const string krmuo::HtmlCodeTokenMap::signEncodation211="&Oacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue212=(unsigned char)212;
       const string krmuo::HtmlCodeTokenMap::signEncodation212="&Ocirc;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue213=(unsigned char)213;
       const string krmuo::HtmlCodeTokenMap::signEncodation213="&Otilde;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue214=(unsigned char)214;
       const string krmuo::HtmlCodeTokenMap::signEncodation214="&Ouml;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue215=(unsigned char)215;
       const string krmuo::HtmlCodeTokenMap::signEncodation215="&times;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue216=(unsigned char)216;
       const string krmuo::HtmlCodeTokenMap::signEncodation216="&Oslash;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue217=(unsigned char)217;
       const string krmuo::HtmlCodeTokenMap::signEncodation217="&Ugrave;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue218=(unsigned char)218;
       const string krmuo::HtmlCodeTokenMap::signEncodation218="&Uacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue219=(unsigned char)219;
       const string krmuo::HtmlCodeTokenMap::signEncodation219="&Ucirc;";

const unsigned char krmuo::HtmlCodeTokenMap::signValue220=(unsigned char)220;
       const string krmuo::HtmlCodeTokenMap::signEncodation220="&Uuml;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue221=(unsigned char)221;
       const string krmuo::HtmlCodeTokenMap::signEncodation221="&Yacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue222=(unsigned char)222;
       const string krmuo::HtmlCodeTokenMap::signEncodation222="&THORN;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue223=(unsigned char)223;
       const string krmuo::HtmlCodeTokenMap::signEncodation223="&szlig;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue224=(unsigned char)224;
       const string krmuo::HtmlCodeTokenMap::signEncodation224="&agrave;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue225=(unsigned char)225;
       const string krmuo::HtmlCodeTokenMap::signEncodation225="&aacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue226=(unsigned char)226;
       const string krmuo::HtmlCodeTokenMap::signEncodation226="&acirc;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue227=(unsigned char)227;
       const string krmuo::HtmlCodeTokenMap::signEncodation227="&atilde;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue228=(unsigned char)228;
       const string krmuo::HtmlCodeTokenMap::signEncodation228="&auml;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue229=(unsigned char)229;
       const string krmuo::HtmlCodeTokenMap::signEncodation229="&aring;";

const unsigned char krmuo::HtmlCodeTokenMap::signValue230=(unsigned char)230;
       const string krmuo::HtmlCodeTokenMap::signEncodation230="&aelig;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue231=(unsigned char)231;
       const string krmuo::HtmlCodeTokenMap::signEncodation231="&ccedil;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue232=(unsigned char)232;
       const string krmuo::HtmlCodeTokenMap::signEncodation232="&egrave;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue233=(unsigned char)233;
       const string krmuo::HtmlCodeTokenMap::signEncodation233="&eacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue234=(unsigned char)234;
       const string krmuo::HtmlCodeTokenMap::signEncodation234="&ecirc;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue235=(unsigned char)235;
       const string krmuo::HtmlCodeTokenMap::signEncodation235="&euml;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue236=(unsigned char)236;
       const string krmuo::HtmlCodeTokenMap::signEncodation236="&igrave;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue237=(unsigned char)237;
       const string krmuo::HtmlCodeTokenMap::signEncodation237="&iacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue238=(unsigned char)238;
       const string krmuo::HtmlCodeTokenMap::signEncodation238="&icirc;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue239=(unsigned char)239;
       const string krmuo::HtmlCodeTokenMap::signEncodation239="&iuml;";

const unsigned char krmuo::HtmlCodeTokenMap::signValue240=(unsigned char)240;
       const string krmuo::HtmlCodeTokenMap::signEncodation240="&eth;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue241=(unsigned char)241;
       const string krmuo::HtmlCodeTokenMap::signEncodation241="&ntilde;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue242=(unsigned char)242;
       const string krmuo::HtmlCodeTokenMap::signEncodation242="&ograve;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue243=(unsigned char)243;
       const string krmuo::HtmlCodeTokenMap::signEncodation243="&oacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue244=(unsigned char)244;
       const string krmuo::HtmlCodeTokenMap::signEncodation244="&ocirc;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue245=(unsigned char)245;
       const string krmuo::HtmlCodeTokenMap::signEncodation245="&otilde;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue246=(unsigned char)246;
       const string krmuo::HtmlCodeTokenMap::signEncodation246="&ouml;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue247=(unsigned char)247;
       const string krmuo::HtmlCodeTokenMap::signEncodation247="&divide;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue248=(unsigned char)248;
       const string krmuo::HtmlCodeTokenMap::signEncodation248="&oslash;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue249=(unsigned char)249;
       const string krmuo::HtmlCodeTokenMap::signEncodation249="&ugrave;";

const unsigned char krmuo::HtmlCodeTokenMap::signValue250=(unsigned char)250;
       const string krmuo::HtmlCodeTokenMap::signEncodation250="&uacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue251=(unsigned char)251;
       const string krmuo::HtmlCodeTokenMap::signEncodation251="&ucirc;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue252=(unsigned char)252;
       const string krmuo::HtmlCodeTokenMap::signEncodation252="&uuml;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue253=(unsigned char)253;
       const string krmuo::HtmlCodeTokenMap::signEncodation253="&yacute;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue254=(unsigned char)254;
       const string krmuo::HtmlCodeTokenMap::signEncodation254="&thorn;";
const unsigned char krmuo::HtmlCodeTokenMap::signValue255=(unsigned char)255;
       const string krmuo::HtmlCodeTokenMap::signEncodation255="&yuml;";

/* --- (3.A) public ------------------------------------------ */

/* :-: (3.A.1) public constructors, inits & operators -:-:-:-: */

/**
 * \brief constructor for class HtmlCodeTokenMap
 *  
 */
krmuo::HtmlCodeTokenMap::HtmlCodeTokenMap()
{
  CodeTokenMap::mClearToken=&mHtmlClearToken;
  CodeTokenMap::mDecoderMap=&mHtmlDecoderMap;
  CodeTokenMap::mEncoderMap=&mHtmlEncoderMap;

  if (mHtmlMapsMustStillBeFilled)
  {    
    mHtmlDecoderMap.insert(make_pair(signEncodationAmp,signValueAmp));
    mHtmlEncoderMap.insert(make_pair(signValueAmp,signEncodationAmp));

    mHtmlDecoderMap.insert(make_pair(signEncodationLt,signValueLt));
    mHtmlEncoderMap.insert(make_pair(signValueLt,signEncodationLt));

    mHtmlDecoderMap.insert(make_pair(signEncodationGt,signValueGt));
    mHtmlEncoderMap.insert(make_pair(signValueGt,signEncodationGt));

    mHtmlDecoderMap.insert(make_pair(signEncodationQuot,signValueQuot));
    mHtmlEncoderMap.insert(make_pair(signValueQuot,signEncodationQuot));

    /* !!! apos doesn't seem to be a real html-tag and should 
     * therefore not be encoded but decoded
     */
    mHtmlDecoderMap.insert(make_pair(signEncodationApos,signValueApos));

    mHtmlEncoderMap.insert(make_pair(signValue160,signEncodation160));
    mHtmlDecoderMap.insert(make_pair(signEncodation160,signValue160));
    mHtmlEncoderMap.insert(make_pair(signValue161,signEncodation161));
    mHtmlDecoderMap.insert(make_pair(signEncodation161,signValue161));
    mHtmlEncoderMap.insert(make_pair(signValue162,signEncodation162));
    mHtmlDecoderMap.insert(make_pair(signEncodation162,signValue162));
    mHtmlEncoderMap.insert(make_pair(signValue163,signEncodation163));
    mHtmlDecoderMap.insert(make_pair(signEncodation163,signValue163));
    mHtmlEncoderMap.insert(make_pair(signValue164,signEncodation164));
    mHtmlDecoderMap.insert(make_pair(signEncodation164,signValue164));
    mHtmlEncoderMap.insert(make_pair(signValue165,signEncodation165));
    mHtmlDecoderMap.insert(make_pair(signEncodation165,signValue165));
    mHtmlEncoderMap.insert(make_pair(signValue166,signEncodation166));
    mHtmlDecoderMap.insert(make_pair(signEncodation166,signValue166));
    mHtmlEncoderMap.insert(make_pair(signValue167,signEncodation167));
    mHtmlDecoderMap.insert(make_pair(signEncodation167,signValue167));
    mHtmlEncoderMap.insert(make_pair(signValue168,signEncodation168));
    mHtmlDecoderMap.insert(make_pair(signEncodation168,signValue168));
    mHtmlEncoderMap.insert(make_pair(signValue169,signEncodation169));
    mHtmlDecoderMap.insert(make_pair(signEncodation169,signValue169));

    mHtmlEncoderMap.insert(make_pair(signValue170,signEncodation170));
    mHtmlDecoderMap.insert(make_pair(signEncodation170,signValue170));
    mHtmlEncoderMap.insert(make_pair(signValue171,signEncodation171));
    mHtmlDecoderMap.insert(make_pair(signEncodation171,signValue171));
    mHtmlEncoderMap.insert(make_pair(signValue172,signEncodation172));
    mHtmlDecoderMap.insert(make_pair(signEncodation172,signValue172));
    mHtmlEncoderMap.insert(make_pair(signValue173,signEncodation173));
    mHtmlDecoderMap.insert(make_pair(signEncodation173,signValue173));
    mHtmlEncoderMap.insert(make_pair(signValue174,signEncodation174));
    mHtmlDecoderMap.insert(make_pair(signEncodation174,signValue174));
    mHtmlEncoderMap.insert(make_pair(signValue175,signEncodation175));
    mHtmlDecoderMap.insert(make_pair(signEncodation175,signValue175));
    mHtmlEncoderMap.insert(make_pair(signValue176,signEncodation176));
    mHtmlDecoderMap.insert(make_pair(signEncodation176,signValue176));
    mHtmlEncoderMap.insert(make_pair(signValue177,signEncodation177));
    mHtmlDecoderMap.insert(make_pair(signEncodation177,signValue177));
    mHtmlEncoderMap.insert(make_pair(signValue178,signEncodation178));
    mHtmlDecoderMap.insert(make_pair(signEncodation178,signValue178));
    mHtmlEncoderMap.insert(make_pair(signValue179,signEncodation179));
    mHtmlDecoderMap.insert(make_pair(signEncodation179,signValue179));
  
    mHtmlEncoderMap.insert(make_pair(signValue180,signEncodation180));
    mHtmlDecoderMap.insert(make_pair(signEncodation180,signValue180));
    mHtmlEncoderMap.insert(make_pair(signValue181,signEncodation181));
    mHtmlDecoderMap.insert(make_pair(signEncodation181,signValue181));
    mHtmlEncoderMap.insert(make_pair(signValue182,signEncodation182));
    mHtmlDecoderMap.insert(make_pair(signEncodation182,signValue182));
    mHtmlEncoderMap.insert(make_pair(signValue183,signEncodation183));
    mHtmlDecoderMap.insert(make_pair(signEncodation183,signValue183));
    mHtmlEncoderMap.insert(make_pair(signValue184,signEncodation184));
    mHtmlDecoderMap.insert(make_pair(signEncodation184,signValue184));
    mHtmlEncoderMap.insert(make_pair(signValue185,signEncodation185));
    mHtmlDecoderMap.insert(make_pair(signEncodation185,signValue185));
    mHtmlEncoderMap.insert(make_pair(signValue186,signEncodation186));
    mHtmlDecoderMap.insert(make_pair(signEncodation186,signValue186));
    mHtmlEncoderMap.insert(make_pair(signValue187,signEncodation187));
    mHtmlDecoderMap.insert(make_pair(signEncodation187,signValue187));
    mHtmlEncoderMap.insert(make_pair(signValue188,signEncodation188));
    mHtmlDecoderMap.insert(make_pair(signEncodation188,signValue188));
    mHtmlEncoderMap.insert(make_pair(signValue189,signEncodation189));
    mHtmlDecoderMap.insert(make_pair(signEncodation189,signValue189));

    mHtmlEncoderMap.insert(make_pair(signValue190,signEncodation190));
    mHtmlDecoderMap.insert(make_pair(signEncodation190,signValue190));
    mHtmlEncoderMap.insert(make_pair(signValue191,signEncodation191));
    mHtmlDecoderMap.insert(make_pair(signEncodation191,signValue191));
    mHtmlEncoderMap.insert(make_pair(signValue192,signEncodation192));
    mHtmlDecoderMap.insert(make_pair(signEncodation192,signValue192));
    mHtmlEncoderMap.insert(make_pair(signValue193,signEncodation193));
    mHtmlDecoderMap.insert(make_pair(signEncodation193,signValue193));
    mHtmlEncoderMap.insert(make_pair(signValue194,signEncodation194));
    mHtmlDecoderMap.insert(make_pair(signEncodation194,signValue194));
    mHtmlEncoderMap.insert(make_pair(signValue195,signEncodation195));
    mHtmlDecoderMap.insert(make_pair(signEncodation195,signValue195));
    mHtmlEncoderMap.insert(make_pair(signValue196,signEncodation196));
    mHtmlDecoderMap.insert(make_pair(signEncodation196,signValue196));
    mHtmlEncoderMap.insert(make_pair(signValue197,signEncodation197));
    mHtmlDecoderMap.insert(make_pair(signEncodation197,signValue197));
    mHtmlEncoderMap.insert(make_pair(signValue198,signEncodation198));
    mHtmlDecoderMap.insert(make_pair(signEncodation198,signValue198));
    mHtmlEncoderMap.insert(make_pair(signValue199,signEncodation199));
    mHtmlDecoderMap.insert(make_pair(signEncodation199,signValue199));

    mHtmlEncoderMap.insert(make_pair(signValue200,signEncodation200));
    mHtmlDecoderMap.insert(make_pair(signEncodation200,signValue200));
    mHtmlEncoderMap.insert(make_pair(signValue201,signEncodation201));
    mHtmlDecoderMap.insert(make_pair(signEncodation201,signValue201));
    mHtmlEncoderMap.insert(make_pair(signValue202,signEncodation202));
    mHtmlDecoderMap.insert(make_pair(signEncodation202,signValue202));
    mHtmlEncoderMap.insert(make_pair(signValue203,signEncodation203));
    mHtmlDecoderMap.insert(make_pair(signEncodation203,signValue203));
    mHtmlEncoderMap.insert(make_pair(signValue204,signEncodation204));
    mHtmlDecoderMap.insert(make_pair(signEncodation204,signValue204));
    mHtmlEncoderMap.insert(make_pair(signValue205,signEncodation205));
    mHtmlDecoderMap.insert(make_pair(signEncodation205,signValue205));
    mHtmlEncoderMap.insert(make_pair(signValue206,signEncodation206));
    mHtmlDecoderMap.insert(make_pair(signEncodation206,signValue206));
    mHtmlEncoderMap.insert(make_pair(signValue207,signEncodation207));
    mHtmlDecoderMap.insert(make_pair(signEncodation207,signValue207));
    mHtmlEncoderMap.insert(make_pair(signValue208,signEncodation208));
    mHtmlDecoderMap.insert(make_pair(signEncodation208,signValue208));
    mHtmlEncoderMap.insert(make_pair(signValue209,signEncodation209));
    mHtmlDecoderMap.insert(make_pair(signEncodation209,signValue209));

    mHtmlEncoderMap.insert(make_pair(signValue210,signEncodation210));
    mHtmlDecoderMap.insert(make_pair(signEncodation210,signValue210));
    mHtmlEncoderMap.insert(make_pair(signValue211,signEncodation211));
    mHtmlDecoderMap.insert(make_pair(signEncodation211,signValue211));
    mHtmlEncoderMap.insert(make_pair(signValue212,signEncodation212));
    mHtmlDecoderMap.insert(make_pair(signEncodation212,signValue212));
    mHtmlEncoderMap.insert(make_pair(signValue213,signEncodation213));
    mHtmlDecoderMap.insert(make_pair(signEncodation213,signValue213));
    mHtmlEncoderMap.insert(make_pair(signValue214,signEncodation214));
    mHtmlDecoderMap.insert(make_pair(signEncodation214,signValue214));
    mHtmlEncoderMap.insert(make_pair(signValue215,signEncodation215));
    mHtmlDecoderMap.insert(make_pair(signEncodation215,signValue215));
    mHtmlEncoderMap.insert(make_pair(signValue216,signEncodation216));
    mHtmlDecoderMap.insert(make_pair(signEncodation216,signValue216));
    mHtmlEncoderMap.insert(make_pair(signValue217,signEncodation217));
    mHtmlDecoderMap.insert(make_pair(signEncodation217,signValue217));
    mHtmlEncoderMap.insert(make_pair(signValue218,signEncodation218));
    mHtmlDecoderMap.insert(make_pair(signEncodation218,signValue218));
    mHtmlEncoderMap.insert(make_pair(signValue219,signEncodation219));
    mHtmlDecoderMap.insert(make_pair(signEncodation219,signValue219));

    mHtmlEncoderMap.insert(make_pair(signValue220,signEncodation220));
    mHtmlDecoderMap.insert(make_pair(signEncodation220,signValue220));
    mHtmlEncoderMap.insert(make_pair(signValue221,signEncodation221));
    mHtmlDecoderMap.insert(make_pair(signEncodation221,signValue221));
    mHtmlEncoderMap.insert(make_pair(signValue222,signEncodation222));
    mHtmlDecoderMap.insert(make_pair(signEncodation222,signValue222));
    mHtmlEncoderMap.insert(make_pair(signValue223,signEncodation223));
    mHtmlDecoderMap.insert(make_pair(signEncodation223,signValue223));
    mHtmlEncoderMap.insert(make_pair(signValue224,signEncodation224));
    mHtmlDecoderMap.insert(make_pair(signEncodation224,signValue224));
    mHtmlEncoderMap.insert(make_pair(signValue225,signEncodation225));
    mHtmlDecoderMap.insert(make_pair(signEncodation225,signValue225));
    mHtmlEncoderMap.insert(make_pair(signValue226,signEncodation226));
    mHtmlDecoderMap.insert(make_pair(signEncodation226,signValue226));
    mHtmlEncoderMap.insert(make_pair(signValue227,signEncodation227));
    mHtmlDecoderMap.insert(make_pair(signEncodation227,signValue227));
    mHtmlEncoderMap.insert(make_pair(signValue228,signEncodation228));
    mHtmlDecoderMap.insert(make_pair(signEncodation228,signValue228));
    mHtmlEncoderMap.insert(make_pair(signValue229,signEncodation229));
    mHtmlDecoderMap.insert(make_pair(signEncodation229,signValue229));

    mHtmlEncoderMap.insert(make_pair(signValue230,signEncodation230));
    mHtmlDecoderMap.insert(make_pair(signEncodation230,signValue230));
    mHtmlEncoderMap.insert(make_pair(signValue231,signEncodation231));
    mHtmlDecoderMap.insert(make_pair(signEncodation231,signValue231));
    mHtmlEncoderMap.insert(make_pair(signValue232,signEncodation232));
    mHtmlDecoderMap.insert(make_pair(signEncodation232,signValue232));
    mHtmlEncoderMap.insert(make_pair(signValue233,signEncodation233));
    mHtmlDecoderMap.insert(make_pair(signEncodation233,signValue233));
    mHtmlEncoderMap.insert(make_pair(signValue234,signEncodation234));
    mHtmlDecoderMap.insert(make_pair(signEncodation234,signValue234));
    mHtmlEncoderMap.insert(make_pair(signValue235,signEncodation235));
    mHtmlDecoderMap.insert(make_pair(signEncodation235,signValue235));
    mHtmlEncoderMap.insert(make_pair(signValue236,signEncodation236));
    mHtmlDecoderMap.insert(make_pair(signEncodation236,signValue236));
    mHtmlEncoderMap.insert(make_pair(signValue237,signEncodation237));
    mHtmlDecoderMap.insert(make_pair(signEncodation237,signValue237));
    mHtmlEncoderMap.insert(make_pair(signValue238,signEncodation238));
    mHtmlDecoderMap.insert(make_pair(signEncodation238,signValue238));
    mHtmlEncoderMap.insert(make_pair(signValue239,signEncodation239));
    mHtmlDecoderMap.insert(make_pair(signEncodation239,signValue239));

    mHtmlEncoderMap.insert(make_pair(signValue240,signEncodation240));
    mHtmlDecoderMap.insert(make_pair(signEncodation240,signValue240));
    mHtmlEncoderMap.insert(make_pair(signValue241,signEncodation241));
    mHtmlDecoderMap.insert(make_pair(signEncodation241,signValue241));
    mHtmlEncoderMap.insert(make_pair(signValue242,signEncodation242));
    mHtmlDecoderMap.insert(make_pair(signEncodation242,signValue242));
    mHtmlEncoderMap.insert(make_pair(signValue243,signEncodation243));
    mHtmlDecoderMap.insert(make_pair(signEncodation243,signValue243));
    mHtmlEncoderMap.insert(make_pair(signValue244,signEncodation244));
    mHtmlDecoderMap.insert(make_pair(signEncodation244,signValue244));
    mHtmlEncoderMap.insert(make_pair(signValue245,signEncodation245));
    mHtmlDecoderMap.insert(make_pair(signEncodation245,signValue245));
    mHtmlEncoderMap.insert(make_pair(signValue246,signEncodation246));
    mHtmlDecoderMap.insert(make_pair(signEncodation246,signValue246));
    mHtmlEncoderMap.insert(make_pair(signValue247,signEncodation247));
    mHtmlDecoderMap.insert(make_pair(signEncodation247,signValue247));
    mHtmlEncoderMap.insert(make_pair(signValue248,signEncodation248));
    mHtmlDecoderMap.insert(make_pair(signEncodation248,signValue248));
    mHtmlEncoderMap.insert(make_pair(signValue249,signEncodation249));
    mHtmlDecoderMap.insert(make_pair(signEncodation249,signValue249));

    mHtmlEncoderMap.insert(make_pair(signValue250,signEncodation250));
    mHtmlDecoderMap.insert(make_pair(signEncodation250,signValue250));
    mHtmlEncoderMap.insert(make_pair(signValue251,signEncodation251));
    mHtmlDecoderMap.insert(make_pair(signEncodation251,signValue251));
    mHtmlEncoderMap.insert(make_pair(signValue252,signEncodation252));
    mHtmlDecoderMap.insert(make_pair(signEncodation252,signValue252));
    mHtmlEncoderMap.insert(make_pair(signValue253,signEncodation253));
    mHtmlDecoderMap.insert(make_pair(signEncodation253,signValue253));
    mHtmlEncoderMap.insert(make_pair(signValue254,signEncodation254));
    mHtmlDecoderMap.insert(make_pair(signEncodation254,signValue254));
    mHtmlEncoderMap.insert(make_pair(signValue255,signEncodation255));
    mHtmlDecoderMap.insert(make_pair(signEncodation255,signValue255));

    mHtmlMapsMustStillBeFilled=false;
  }
}


/**
 *  \brief class-destructor
 *
 *  class-destructor which gets no parameters and returns nothing
 */
krmuo::HtmlCodeTokenMap::~HtmlCodeTokenMap()
{
}

/* :-: (3.A.2) public getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */


/* :-: (3.A.3) public setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (3.A.4) public others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/* &&& (4) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&&      definitions of static global variables &&&&&&&&&&& */
const string krmuo::XmlCodeTokenMap::mXmlClearToken="";
map<string,unsigned char> krmuo::XmlCodeTokenMap::mXmlDecoderMap;
map<unsigned char,string> krmuo::XmlCodeTokenMap::mXmlEncoderMap;
bool krmuo::XmlCodeTokenMap::mXmlMapsMustStillBeFilled=true;

const unsigned char krmuo::XmlCodeTokenMap::signValueAmp=(unsigned char)'&';
       const string krmuo::XmlCodeTokenMap::signEncodationAmp="&amp;";
  
const unsigned char krmuo::XmlCodeTokenMap::signValueLt=(unsigned char)'<';
       const string krmuo::XmlCodeTokenMap::signEncodationLt="&lt;";

const unsigned char krmuo::XmlCodeTokenMap::signValueGt=(unsigned char)'>';
       const string krmuo::XmlCodeTokenMap::signEncodationGt="&gt;";

const unsigned char krmuo::XmlCodeTokenMap::signValueApos=(unsigned char)'\'';
       const string krmuo::XmlCodeTokenMap::signEncodationApos="&apos;";

const unsigned char krmuo::XmlCodeTokenMap::signValueQuot=(unsigned char)'\"';
       const string krmuo::XmlCodeTokenMap::signEncodationQuot="&quot;";

/* --- (4.A) - public ----------------------------------------- */
/* :-: (4.A.1) public constructors, inits & operators :-:-:-:-: */
/**
 * \brief constructor for class XmlCodeTokenMap
 */
krmuo::XmlCodeTokenMap::XmlCodeTokenMap()
{
  CodeTokenMap::mClearToken=&mXmlClearToken;
  CodeTokenMap::mDecoderMap=&mXmlDecoderMap;
  CodeTokenMap::mEncoderMap=&mXmlEncoderMap;
  if (mXmlMapsMustStillBeFilled)
  {
    
    mXmlDecoderMap.insert(make_pair(signEncodationAmp,signValueAmp));
    mXmlEncoderMap.insert(make_pair(signValueAmp,signEncodationAmp));

    mXmlDecoderMap.insert(make_pair(signEncodationLt,signValueLt));
    mXmlEncoderMap.insert(make_pair(signValueLt,signEncodationLt));

    mXmlDecoderMap.insert(make_pair(signEncodationGt,signValueGt));
    mXmlEncoderMap.insert(make_pair(signValueGt,signEncodationGt));

    mXmlDecoderMap.insert(make_pair(signEncodationApos,signValueApos));
    mXmlEncoderMap.insert(make_pair(signValueApos,signEncodationApos));

    mXmlDecoderMap.insert(make_pair(signEncodationQuot,signValueQuot));
    mXmlEncoderMap.insert(make_pair(signValueQuot,signEncodationQuot));
    
    mXmlMapsMustStillBeFilled=false;
  }
}

/**
 * \brief class-destructor
 */
krmuo::XmlCodeTokenMap::~XmlCodeTokenMap()
{
}

/* &&& (5) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&&     methods for class &&& */
const string krmuo::UrlCodeTokenMap::mUrlClearToken="";
map<string,unsigned char> krmuo::UrlCodeTokenMap::mUrlDecoderMap;
map<unsigned char,string> krmuo::UrlCodeTokenMap::mUrlEncoderMap;
bool krmuo::UrlCodeTokenMap::mUrlMapsMustStillBeFilled=true;

const unsigned char krmuo::UrlCodeTokenMap::signValueSemicolon=(unsigned char)';';
       const string krmuo::UrlCodeTokenMap::signEncodationSemicolon="%3B";

const unsigned char krmuo::UrlCodeTokenMap::signValueSlash=(unsigned char)'/';
       const string krmuo::UrlCodeTokenMap::signEncodationSlash="%2F";

const unsigned char krmuo::UrlCodeTokenMap::signValueQuest=(unsigned char)'?';
       const string krmuo::UrlCodeTokenMap::signEncodationQuest="%3F";

const unsigned char krmuo::UrlCodeTokenMap::signValueColon=(unsigned char)':';
       const string krmuo::UrlCodeTokenMap::signEncodationColon="%3A";

const unsigned char krmuo::UrlCodeTokenMap::signValueAtsign=(unsigned char)'@';
       const string krmuo::UrlCodeTokenMap::signEncodationAtsign="%40";

const unsigned char krmuo::UrlCodeTokenMap::signValueEqual=(unsigned char)'=';
       const string krmuo::UrlCodeTokenMap::signEncodationEqual="%3D";
 
const unsigned char krmuo::UrlCodeTokenMap::signValueAmpercent=(unsigned char)'&';
       const string krmuo::UrlCodeTokenMap::signEncodationAmpercent="%26";
 
const unsigned char krmuo::UrlCodeTokenMap::signValueSpace=(unsigned char)' ';
       const string krmuo::UrlCodeTokenMap::signEncodationSpace="%20";

const unsigned char krmuo::UrlCodeTokenMap::signValueLt=(unsigned char)'<';
       const string krmuo::UrlCodeTokenMap::signEncodationLt="%3C";

const unsigned char krmuo::UrlCodeTokenMap::signValueGt=(unsigned char)'>';
       const string krmuo::UrlCodeTokenMap::signEncodationGt="%3E";

const unsigned char krmuo::UrlCodeTokenMap::signValueQuote=(unsigned char)'"';
       const string krmuo::UrlCodeTokenMap::signEncodationQuote="%22";

const unsigned char krmuo::UrlCodeTokenMap::signValueGadder=(unsigned char)'#';
       const string krmuo::UrlCodeTokenMap::signEncodationGadder="%23";

const unsigned char krmuo::UrlCodeTokenMap::signValuePercent=(unsigned char)'%';
       const string krmuo::UrlCodeTokenMap::signEncodationPercent="%25";

const unsigned char krmuo::UrlCodeTokenMap::signValueLsBracket=(unsigned char)'[';
       const string krmuo::UrlCodeTokenMap::signEncodationLsBracket="%5B";

const unsigned char krmuo::UrlCodeTokenMap::signValueRsBracket=(unsigned char)']';
       const string krmuo::UrlCodeTokenMap::signEncodationRsBracket="%5D";
 
const unsigned char krmuo::UrlCodeTokenMap::signValueLBrace=(unsigned char)'{';
       const string krmuo::UrlCodeTokenMap::signEncodationLBrace="%7B";

const unsigned char krmuo::UrlCodeTokenMap::signValueRBrace=(unsigned char)'}';
       const string krmuo::UrlCodeTokenMap::signEncodationRBrace="%7D";

const unsigned char krmuo::UrlCodeTokenMap::signValueVBar=(unsigned char)'|';
       const string krmuo::UrlCodeTokenMap::signEncodationVBar="%7C";

const unsigned char krmuo::UrlCodeTokenMap::signValueBackSlash=(unsigned char)'\\';
       const string krmuo::UrlCodeTokenMap::signEncodationBackSlash="%5C";

const unsigned char krmuo::UrlCodeTokenMap::signValueCaret=(unsigned char)'^';
       const string krmuo::UrlCodeTokenMap::signEncodationCaret="%5E";

const unsigned char krmuo::UrlCodeTokenMap::signValueTilde=(unsigned char)'~';
       const string krmuo::UrlCodeTokenMap::signEncodationTilde="%7E";

const unsigned char krmuo::UrlCodeTokenMap::signValueBackQuote=(unsigned char)'`';
       const string krmuo::UrlCodeTokenMap::signEncodationBackQuote="%60";
     
const unsigned char krmuo::UrlCodeTokenMap::signValuePlusSign=(unsigned char)'+';
       const string krmuo::UrlCodeTokenMap::signEncodationPlusSign="%2B";
       
const unsigned char krmuo::UrlCodeTokenMap::signValueBlank=(unsigned char)' ';
       const string krmuo::UrlCodeTokenMap::signEncodationBlank="+";


/* --- (5.A) - public ----------------------------------------- */
/* :-: (5.A.1) public constructors, inits & operators :-:-:-:-: */
/**
 * \brief constructor for class XmlCodeTokenMap
 */
krmuo::UrlCodeTokenMap::UrlCodeTokenMap()
{
  CodeTokenMap::mClearToken=&mUrlClearToken;
  CodeTokenMap::mDecoderMap=&mUrlDecoderMap;
  CodeTokenMap::mEncoderMap=&mUrlEncoderMap;
  if (mUrlMapsMustStillBeFilled)
  {
   
    mUrlDecoderMap.insert(make_pair(signEncodationSemicolon,signValueSemicolon));
    mUrlEncoderMap.insert(make_pair(signValueSemicolon,signEncodationSemicolon));

    mUrlDecoderMap.insert(make_pair(signEncodationSlash,signValueSlash));
    mUrlEncoderMap.insert(make_pair(signValueSlash,signEncodationSlash));

    mUrlDecoderMap.insert(make_pair(signEncodationQuest,signValueQuest));
    mUrlEncoderMap.insert(make_pair(signValueQuest,signEncodationQuest));
    
    mUrlDecoderMap.insert(make_pair(signEncodationColon,signValueColon));
    mUrlEncoderMap.insert(make_pair(signValueColon,signEncodationColon));
    
    mUrlDecoderMap.insert(make_pair(signEncodationAtsign,signValueAtsign));
    mUrlEncoderMap.insert(make_pair(signValueAtsign,signEncodationAtsign));
    
    mUrlDecoderMap.insert(make_pair(signEncodationEqual,signValueEqual));
    mUrlEncoderMap.insert(make_pair(signValueEqual,signEncodationEqual));
    
    mUrlDecoderMap.insert(make_pair(signEncodationAmpercent,signValueAmpercent));
    mUrlEncoderMap.insert(make_pair(signValueAmpercent,signEncodationAmpercent));

    mUrlDecoderMap.insert(make_pair(signEncodationSpace,signValueSpace));
    mUrlEncoderMap.insert(make_pair(signValueSpace,signEncodationSpace));

    mUrlDecoderMap.insert(make_pair(signEncodationLt,signValueLt));
    mUrlEncoderMap.insert(make_pair(signValueLt,signEncodationLt));
    
    mUrlDecoderMap.insert(make_pair(signEncodationGt,signValueGt));
    mUrlEncoderMap.insert(make_pair(signValueGt,signEncodationGt));
    
    mUrlDecoderMap.insert(make_pair(signEncodationQuote,signValueQuote));
    mUrlEncoderMap.insert(make_pair(signValueQuote,signEncodationQuote));
    
    mUrlDecoderMap.insert(make_pair(signEncodationGadder,signValueGadder));
    mUrlEncoderMap.insert(make_pair(signValueGadder,signEncodationGadder));

    mUrlDecoderMap.insert(make_pair(signEncodationPercent,signValuePercent));
    mUrlEncoderMap.insert(make_pair(signValuePercent,signEncodationPercent));

    mUrlDecoderMap.insert(make_pair(signEncodationLsBracket,signValueLsBracket));
    mUrlEncoderMap.insert(make_pair(signValueLsBracket,signEncodationLsBracket));
    
    mUrlDecoderMap.insert(make_pair(signEncodationRsBracket,signValueRsBracket));
    mUrlEncoderMap.insert(make_pair(signValueRsBracket,signEncodationRsBracket));
    
    mUrlDecoderMap.insert(make_pair(signEncodationLBrace,signValueLBrace));
    mUrlEncoderMap.insert(make_pair(signValueLBrace,signEncodationLBrace));
    
    mUrlDecoderMap.insert(make_pair(signEncodationRBrace,signValueRBrace));
    mUrlEncoderMap.insert(make_pair(signValueRBrace,signEncodationRBrace));
    
    mUrlDecoderMap.insert(make_pair(signEncodationVBar,signValueVBar));
    mUrlEncoderMap.insert(make_pair(signValueVBar,signEncodationVBar));
   
    mUrlDecoderMap.insert(make_pair(signEncodationBackSlash,signValueBackSlash));
    mUrlEncoderMap.insert(make_pair(signValueBackSlash,signEncodationBackSlash));
    
    mUrlDecoderMap.insert(make_pair(signEncodationCaret,signValueCaret));
    mUrlEncoderMap.insert(make_pair(signValueCaret,signEncodationCaret));
    
    mUrlDecoderMap.insert(make_pair(signEncodationTilde,signValueTilde));
    mUrlEncoderMap.insert(make_pair(signValueTilde,signEncodationTilde));
    
    mUrlDecoderMap.insert(make_pair(signEncodationPlusSign,signValuePlusSign));
    mUrlEncoderMap.insert(make_pair(signValuePlusSign,signEncodationPlusSign));

    mUrlDecoderMap.insert(make_pair(signEncodationBlank,signValueBlank));
    mUrlEncoderMap.insert(make_pair(signValueBlank,signEncodationBlank));
     
    mUrlMapsMustStillBeFilled=false;
  }
}
/**
 * \brief class-destructor
 */
krmuo::UrlCodeTokenMap::~UrlCodeTokenMap()
{
}
/* :-: (5.A.2) public getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (5.A.3) public setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (5.A.4) public others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/* --- (5.B) - private ---------------------------------------- */
/* :-: (5.B.1) private constructors, inits & operators -:-:-:-: */
/* :-: (5.B.2) private getter -:-:-:-:-:-:-:-:-:-::-:-:-:-:-:-: */
/* :-: (5.B.3) private setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (5.B.4) private others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/* --- (5.C) - protected -------------------------------------- */
/* :-: (5.C.1) protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (5.C.2) protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (5.C.3) protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
