/*
 * Copyright (C) 2001 Karsten Reincke <karsten.reincke@karubik.de>
 *  
 * This file is part of of the software-project MRPROJEXT.
 * 
 * MRPROJEXT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * MRPROJEXT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 * file <mrprjExtractor.cc> version <#1.3.1#> of project <MrProjext>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include <string>
#include <muo.h>
#include <popt.h>
#include <time.h>
#include "mrprjContent.h"
#include "cmdline.h"

int main (int argn, char * argv[])
{
  gengetopt_args_info args_info;
   
  if (cmdline_parser(argn,argv, &args_info)!=0)
  {
    cerr << "grrr: gengetopt-args can't be parsed\n";
    exit(-1);
  }
 
  int messLevel=SM;
  if (args_info.debug_flag)
    messLevel=HM;
 
  krmuo::LoggingUnit faultLogging(messLevel,stderr);

  
  string xmlFileName="stdin";
  FILE *fp;  
  if (args_info.dbFile_given)
  {
    xmlFileName=args_info.dbFile_arg;
    fp=fopen(args_info.dbFile_arg,"r");
    if (!(fp))
    {
      faultLogging.makeEntry("Can't open/found inputfile",xmlFileName,SIGNAL_MESSAGE);
      exit(-1);
    }
  }
  else
  {
    fp=stdin;
  }
  
  char sign;
  string xmlFileContent;
  do
  {
    sign=fgetc(fp);
    if ((sign!=EOF))
      xmlFileContent+=sign;
  }
  while(sign!=EOF);
  
  if (fp!=stdin)
    fclose(fp);

  string resourceGroupNameString="all";
  string resourceNameString="all";
  string timeSlotEndString="21001231T240000";  
  string timeSlotStartString="19700101T000000";  
  string surveyReferenceDayString="20020535T256363";

  if (args_info.rwHtmlSurvey_flag)
  {
    if (args_info.evaluationDate_given)
      surveyReferenceDayString=args_info.evaluationDate_arg;
    else
    {
      time_t now;
      
      if (time(&now))
      {
      	struct tm* tp;
	char timeStamp[20];
	tp=localtime(&now);
	strftime(timeStamp,20,"%Y%m%dT%H%M%S",tp);
      	surveyReferenceDayString=timeStamp;
      }
    }
  }
  else
  {
    if (args_info.timeSlotStart_given)
      timeSlotStartString=args_info.timeSlotStart_arg;

    if (args_info.timeSlotEnd_given)
      timeSlotEndString=args_info.timeSlotEnd_arg;
  
    if (args_info.person_given)
      resourceNameString=args_info.person_arg;
  
    if (args_info.group_given)
      resourceGroupNameString=args_info.group_arg; 
  }

  if (args_info.debug_flag)
  {
    string resourceMessage=("concerning the persons/resources: "+resourceNameString);
    if (resourceGroupNameString!="all")
      resourceMessage=("concerning all persons of group:" +resourceGroupNameString);

    string doing= "extracting data from " +
      	      	xmlFileName + 
		"\n    " + resourceMessage +
		"\n    and the timeslot " +
		timeSlotStartString +
		" - " +
		timeSlotEndString +
		"\n";
		
    faultLogging.makeEntry(doing,SIGNAL_MESSAGE);
  }

  mrprj::TaskField mrProjectTaskField(faultLogging);

  if (!(mrProjectTaskField.fillTaskFieldByMrPrjXmlFileContent(faultLogging,xmlFileContent)))
  {
    faultLogging.makeEntry("The given mrprjBaseFile can't be analyzed",xmlFileName,SIGNAL_MESSAGE);
    return -1;
  }

  if (args_info.rwHtmlSurvey_flag)
  {
    mrProjectTaskField.printTaskHtmlSurvey(surveyReferenceDayString);
    return 0;
  }
  
  
  if (args_info.group_given)
  {
    mrProjectTaskField.limitToResourceGroup(resourceGroupNameString);
  }
  else
  {
    if (args_info.person_given)
      mrProjectTaskField.limitToResource(resourceNameString);
  }

  mrProjectTaskField.limitToTimeSlot(timeSlotStartString,timeSlotEndString);

  if ((args_info.onlyOpenTasks_flag)||(args_info.only_open_flag))
    mrProjectTaskField.limitToOpenTasks();
  
  mrProjectTaskField.limitToThoseWithActiveDaughters();
  
  if (((bool)args_info.rwOnlyLeafs_flag)||((bool)args_info.only_leafs_flag))
  {
    if (((bool)args_info.rwMemoFile_flag)||((bool)args_info.output_memo_flag))
      mrProjectTaskField.printTaskFieldAsLeafListMemo((bool)args_info.rwIso8601_flag||(bool)args_info.iso_dates_flag);
    else if (((bool)args_info.rwPilotFile_flag)||((bool)args_info.output_pilot_flag))
      mrProjectTaskField.printTaskFieldAsPilotLeafTodo(); 
    else if (((bool)args_info.rwVcardFile_flag)||((bool)args_info.output_vcard_flag))
      mrProjectTaskField.printTaskFieldAsLeafVTodo();
    else if (((bool)args_info.rwYankFile_flag)||((bool)args_info.output_yank_flag))
      mrProjectTaskField.printTaskFieldAsLeafYankTodo();
    else if (((bool)args_info.rwCslFile_flag)||((bool)args_info.output_csl_flag))
      mrProjectTaskField.printTaskFieldAsLeafCslTodo();
    else
      mrProjectTaskField.printTaskFieldAsMrPrjXmlFile(); 
  }
  else
  {
    if (((bool)args_info.rwMemoFile_flag)||((bool)args_info.output_memo_flag))
      mrProjectTaskField.printTaskFieldAsTreeMemo((bool)args_info.rwIso8601_flag||(bool)args_info.iso_dates_flag);
    else if (((bool)args_info.rwPilotFile_flag)||((bool)args_info.output_pilot_flag))
      mrProjectTaskField.printTaskFieldAsPilotTreeTodo();    
    else if (((bool)args_info.rwVcardFile_flag)||((bool)args_info.output_vcard_flag))
      mrProjectTaskField.printTaskFieldAsTreeVTodo();
    else if (((bool)args_info.rwYankFile_flag)||((bool)args_info.output_yank_flag))
      mrProjectTaskField.printTaskFieldAsTreeYankTodo();
    else if (((bool)args_info.rwCslFile_flag)||((bool)args_info.output_csl_flag))
      mrProjectTaskField.printTaskFieldAsTreeCslTodo();
    else
      mrProjectTaskField.printTaskFieldAsMrPrjXmlFile(); 
  }

  return 0;
}
